@@ -10,6 +10,11 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+import typing as ty
+
+from keystoneauth1 import adapter
+import typing_extensions as ty_ext
+
 from openstack import exceptions
 from openstack import resource
 from openstack import utils
@@ -91,11 +96,16 @@ class Flavor(resource.Resource):
     @classmethod
     def list(
         cls,
-        session,
-        paginated=True,
-        base_path='/flavors/detail',
-        **params,
-    ):
+        session: adapter.Adapter,
+        paginated: bool = True,
+        base_path: str | None = '/flavors/detail',
+        allow_unknown_params: bool = False,
+        *,
+        microversion: str | None = None,
+        headers: dict[str, str] | None = None,
+        max_items: int | None = None,
+        **params: ty.Any,
+    ) -> ty.Generator[ty_ext.Self, None, None]:
         # Find will invoke list when name was passed. Since we want to return
         # flavor with details (same as direct get) we need to swap default here
         # and list with "/flavors" if no details explicitely requested
@@ -103,8 +113,15 @@ class Flavor(resource.Resource):
             # is_public is ternary - None means give all flavors.
             # Force it to string to avoid requests skipping it.
             params['is_public'] = 'None'
-        return super(Flavor, cls).list(
-            session, paginated=paginated, base_path=base_path, **params
+        return super().list(
+            session,
+            paginated=paginated,
+            base_path=base_path,
+            allow_unknown_params=allow_unknown_params,
+            microversion=microversion,
+            headers=headers,
+            max_items=max_items,
+            **params,
         )
 
     def _action(self, session, body, microversion=None):
@@ -120,12 +137,22 @@ class Flavor(resource.Resource):
         return response
 
     def add_tenant_access(self, session, tenant):
-        """Adds flavor access to a tenant and flavor."""
+        """Adds flavor access to a tenant and flavor.
+
+        :param session: The session to use for making this request.
+        :param tenant:
+        :returns: None
+        """
         body = {'addTenantAccess': {'tenant': tenant}}
         self._action(session, body)
 
     def remove_tenant_access(self, session, tenant):
-        """Removes flavor access to a tenant and flavor."""
+        """Removes flavor access to a tenant and flavor.
+
+        :param session: The session to use for making this request.
+        :param tenant:
+        :returns: None
+        """
         body = {'removeTenantAccess': {'tenant': tenant}}
         self._action(session, body)
 
@@ -133,9 +160,10 @@ class Flavor(resource.Resource):
         """Lists tenants who have access to a private flavor
 
         By default, only administrators can manage private flavor access. A
-        private flavor has is_public set to false while a public flavor has
-        is_public set to true.
+        private flavor has ``is_public`` set to false while a public flavor has
+        ``is_public`` set to true.
 
+        :param session: The session to use for making this request.
         :return: List of dicts with flavor_id and tenant_id attributes
         """
         url = utils.urljoin(Flavor.base_path, self.id, 'os-flavor-access')
@@ -144,13 +172,16 @@ class Flavor(resource.Resource):
         return response.json().get('flavor_access', [])
 
     def fetch_extra_specs(self, session):
-        """Fetch extra_specs of the flavor
+        """Fetch extra specs of the flavor
 
-        Starting with 2.61 extra_specs are returned with the flavor details,
+        Starting with 2.61 extra specs are returned with the flavor details,
         before that a separate call is required.
+
+        :param session: The session to use for making this request.
+        :returns: The updated flavor.
         """
         url = utils.urljoin(Flavor.base_path, self.id, 'os-extra_specs')
-        microversion = self._get_microversion(session, action='fetch')
+        microversion = self._get_microversion(session)
         response = session.get(url, microversion=microversion)
         exceptions.raise_from_response(response)
         specs = response.json().get('extra_specs', {})
@@ -158,9 +189,14 @@ class Flavor(resource.Resource):
         return self
 
     def create_extra_specs(self, session, specs):
-        """Creates extra specs for a flavor"""
+        """Creates extra specs for a flavor.
+
+        :param session: The session to use for making this request.
+        :param specs:
+        :returns: The updated flavor.
+        """
         url = utils.urljoin(Flavor.base_path, self.id, 'os-extra_specs')
-        microversion = self._get_microversion(session, action='create')
+        microversion = self._get_microversion(session)
         response = session.post(
             url, json={'extra_specs': specs}, microversion=microversion
         )
@@ -170,18 +206,29 @@ class Flavor(resource.Resource):
         return self
 
     def get_extra_specs_property(self, session, prop):
-        """Get individual extra_spec property"""
+        """Get an individual extra spec property.
+
+        :param session: The session to use for making this request.
+        :param prop: The property to fetch.
+        :returns: The value of the property if it exists, else ``None``.
+        """
         url = utils.urljoin(Flavor.base_path, self.id, 'os-extra_specs', prop)
-        microversion = self._get_microversion(session, action='fetch')
+        microversion = self._get_microversion(session)
         response = session.get(url, microversion=microversion)
         exceptions.raise_from_response(response)
         val = response.json().get(prop)
         return val
 
     def update_extra_specs_property(self, session, prop, val):
-        """Update An Extra Spec For A Flavor"""
+        """Update an extra spec for a flavor.
+
+        :param session: The session to use for making this request.
+        :param prop: The property to update.
+        :param val: The value to update with.
+        :returns: The updated value of the property.
+        """
         url = utils.urljoin(Flavor.base_path, self.id, 'os-extra_specs', prop)
-        microversion = self._get_microversion(session, action='commit')
+        microversion = self._get_microversion(session)
         response = session.put(
             url, json={prop: val}, microversion=microversion
         )
@@ -190,11 +237,17 @@ class Flavor(resource.Resource):
         return val
 
     def delete_extra_specs_property(self, session, prop):
-        """Delete An Extra Spec For A Flavor"""
+        """Delete an extra spec for a flavor.
+
+        :param session: The session to use for making this request.
+        :param prop: The property to delete.
+        :returns: None
+        """
         url = utils.urljoin(Flavor.base_path, self.id, 'os-extra_specs', prop)
-        microversion = self._get_microversion(session, action='delete')
+        microversion = self._get_microversion(session)
         response = session.delete(url, microversion=microversion)
         exceptions.raise_from_response(response)
 
 
+# TODO(stephenfin): Deprecate this for removal in 2.0
 FlavorDetail = Flavor
