@@ -12,10 +12,13 @@
 
 import typing as ty
 
+from openstack.dns.v2 import blacklist as _blacklist
 from openstack.dns.v2 import floating_ip as _fip
 from openstack.dns.v2 import limit as _limit
+from openstack.dns.v2 import quota as _quota
 from openstack.dns.v2 import recordset as _rs
 from openstack.dns.v2 import service_status as _svc_status
+from openstack.dns.v2 import tld as _tld
 from openstack.dns.v2 import tsigkey as _tsigkey
 from openstack.dns.v2 import zone as _zone
 from openstack.dns.v2 import zone_export as _zone_export
@@ -29,8 +32,10 @@ from openstack import resource
 
 class Proxy(proxy.Proxy):
     _resource_registry = {
+        "blacklist": _blacklist.Blacklist,
         "floating_ip": _fip.FloatingIP,
         "limits": _limit.Limit,
+        "quota": _quota.Quota,
         "recordset": _rs.Recordset,
         "service_status": _svc_status.ServiceStatus,
         "zone": _zone.Zone,
@@ -40,6 +45,7 @@ class Proxy(proxy.Proxy):
         "zone_nameserver": _zone_nameserver.ZoneNameserver,
         "zone_share": _zone_share.ZoneShare,
         "zone_transfer_request": _zone_transfer.ZoneTransferRequest,
+        "tld": _tld.TLD,
     }
 
     # ======== Zones ========
@@ -695,6 +701,60 @@ class Proxy(proxy.Proxy):
         """
         return self._list(_limit.Limit, **query)
 
+    # ======== Quotas ========
+    def quotas(self, **query):
+        """Return a generator of quotas
+
+        :param dict query: Optional query parameters to be sent to limit the
+            resources being returned.
+
+        :returns: A generator of quota objects
+        :rtype: :class:`~openstack.dns.v2.quota.Quota`
+        """
+        return self._list(_quota.Quota, **query)
+
+    def get_quota(self, quota):
+        """Get a quota
+
+        :param quota: The value can be the ID of a quota or a
+            :class:`~openstack.dns.v2.quota.Quota` instance.
+            The ID of a quota is the same as the project ID for the quota.
+
+        :returns: One :class:`~openstack.dns.v2.quota.Quota`
+        :raises: :class:`~openstack.exceptions.ResourceNotFound`
+        """
+        return self._get(_quota.Quota, quota)
+
+    def update_quota(self, quota, **attrs):
+        """Update a quota
+
+        :param quota: Either the ID of a quota or a
+            :class:`~openstack.dns.v2.quota.Quota` instance. The ID of a quota
+            is the same as the project ID for the quota.
+        :param dict attrs: The attributes to update on the quota represented
+            by ``quota``.
+
+        :returns: The updated quota
+        :rtype: :class:`~openstack.dns.v2.quota.Quota`
+        """
+        return self._update(_quota.Quota, quota, **attrs)
+
+    def delete_quota(self, quota, ignore_missing=True):
+        """Delete a quota (i.e. reset to the default quota)
+
+        :param quota: The value can be the ID of a quota or a
+            :class:`~openstack.dns.v2.quota.Quota` instance.
+            The ID of a quota is the same as the project ID for the quota.
+        :param bool ignore_missing: When set to ``False``,
+            :class:`~openstack.exceptions.ResourceNotFound` will be raised when
+            the quota does not exist.
+            When set to ``True``, no exception will be set when attempting to
+            delete a nonexistent quota.
+
+        :returns: ``None``
+        """
+        return self._delete(_quota.Quota, quota, ignore_missing=ignore_missing)
+
     # ======== Service Statuses ========
     def service_statuses(self):
         """Retrieve a generator of service statuses
@@ -715,109 +775,86 @@ class Proxy(proxy.Proxy):
         """
         return self._get(_svc_status.ServiceStatus, service)
 
-    # ========== Utilities ==========
-    def wait_for_status(
-        self,
-        res: resource.ResourceT,
-        status: str,
-        failures: ty.Optional[list[str]] = None,
-        interval: ty.Union[int, float, None] = 2,
-        wait: ty.Optional[int] = None,
-        attribute: str = 'status',
-        callback: ty.Optional[ty.Callable[[int], None]] = None,
-    ) -> resource.ResourceT:
-        """Wait for the resource to be in a particular status.
+    # ======== TLDs ========
+    def tlds(self, **query):
+        """Retrieve a generator of tlds
 
-        :param session: The session to use for making this request.
-        :param resource: The resource to wait on to reach the status. The
-            resource must have a status attribute specified via ``attribute``.
-        :param status: Desired status of the resource.
-        :param failures: Statuses that would indicate the transition
-            failed such as 'ERROR'. Defaults to ['ERROR'].
-        :param interval: Number of seconds to wait between checks.
-        :param wait: Maximum number of seconds to wait for transition.
-            Set to ``None`` to wait forever.
-        :param attribute: Name of the resource attribute that contains the
-            status.
-        :param callback: A callback function. This will be called with a single
-            value, progress. This is API specific but is generally a percentage
-            value from 0-100.
+        :param dict query: Optional query parameters to be sent to limit the
+            resources being returned.
 
-        :return: The updated resource.
-        :raises: :class:`~openstack.exceptions.ResourceTimeout` if the
-            transition to status failed to occur in ``wait`` seconds.
-        :raises: :class:`~openstack.exceptions.ResourceFailure` if the resource
-            transitioned to one of the states in ``failures``.
-        :raises: :class:`~AttributeError` if the resource does not have a
-            ``status`` attribute
+            * `name`: TLD Name field.
+
+        :returns: A generator of tld
+            :class:`~openstack.dns.v2.tld.TLD` instances.
         """
-        return resource.wait_for_status(
-            self, res, status, failures, interval, wait, attribute, callback
+        return self._list(_tld.TLD, **query)
+
+    def create_tld(self, **attrs):
+        """Create a new tld from attributes
+
+        :param dict attrs: Keyword arguments which will be used to create
+            a :class:`~openstack.dns.v2.tld.TLD`,
+            comprised of the properties on the TLD class.
+        :returns: The results of TLD creation.
+        :rtype: :class:`~openstack.dns.v2.tld.TLD`
+        """
+        return self._create(_tld.TLD, prepend_key=False, **attrs)
+
+    def get_tld(self, tld):
+        """Get a tld
+
+        :param tld: The value can be the ID of a tld
+            or a :class:`~openstack.dns.v2.tld.TLD` instance.
+        :returns: tld instance.
+        :rtype: :class:`~openstack.dns.v2.tld.TLD`
+        """
+        return self._get(_tld.TLD, tld)
+
+    def delete_tld(self, tld, ignore_missing=True):
+        """Delete a tld
+
+        :param tld: The value can be the ID of a tld
+            or a :class:`~openstack.dns.v2.tld.TLD` instance.
+        :param bool ignore_missing: When set to ``False``
+            :class:`~openstack.exceptions.NotFoundException` will be raised when
+            the tld does not exist.
+            When set to ``True``, no exception will be set when attempting to
+            delete a nonexistent tld.
+
+        :returns: TLD been deleted
+        :rtype: :class:`~openstack.dns.v2.tld.TLD`
+        """
+        return self._delete(
+            _tld.TLD,
+            tld,
+            ignore_missing=ignore_missing,
         )
 
-    def wait_for_delete(
-        self,
-        res: resource.ResourceT,
-        interval: int = 2,
-        wait: int = 120,
-        callback: ty.Optional[ty.Callable[[int], None]] = None,
-    ) -> resource.ResourceT:
-        """Wait for a resource to be deleted.
+    def update_tld(self, tld, **attrs):
+        """Update tld attributes
 
-        :param res: The resource to wait on to be deleted.
-        :param interval: Number of seconds to wait before to consecutive
-            checks.
-        :param wait: Maximum number of seconds to wait before the change.
-        :param callback: A callback function. This will be called with a single
-            value, progress, which is a percentage value from 0-100.
+        :param tld: The id or an instance of
+            :class:`~openstack.dns.v2.tld.TLD`.
+        :param dict attrs: attributes for update on
+            :class:`~openstack.dns.v2.tld.TLD`.
 
-        :returns: The resource is returned on success.
-        :raises: :class:`~openstack.exceptions.ResourceTimeout` if transition
-            to delete failed to occur in the specified seconds.
+        :rtype: :class:`~openstack.dns.v2.tld.TLD`
         """
-        return resource.wait_for_delete(self, res, interval, wait, callback)
+        return self._update(_tld.TLD, tld, **attrs)
 
-    def _get_cleanup_dependencies(self):
-        # DNS may depend on floating ip
-        return {'dns': {'before': ['network']}}
+    def find_tld(self, name_or_id, ignore_missing=True):
+        """Find a single tld
 
-    def _service_cleanup(
-        self,
-        dry_run=True,
-        client_status_queue=False,
-        identified_resources=None,
-        filters=None,
-        resource_evaluation_fn=None,
-        skip_resources=None,
-    ):
-        if not self.should_skip_resource_cleanup("zone", skip_resources):
-            # Delete all zones
-            for obj in self.zones():
-                self._service_cleanup_del_res(
-                    self.delete_zone,
-                    obj,
-                    dry_run=dry_run,
-                    client_status_queue=client_status_queue,
-                    identified_resources=identified_resources,
-                    filters=filters,
-                    resource_evaluation_fn=resource_evaluation_fn,
-                )
+        :param name_or_id: The name or ID of a tld
+        :param bool ignore_missing: When set to ``False``
+            :class:`~openstack.exceptions.NotFoundException` will be raised
+            when the tld does not exist.
+            When set to ``True``, no exception will be set when attempting
+            to delete a nonexistent tld.
 
-        if not self.should_skip_resource_cleanup(
-            "floating_ip", skip_resources
-        ):
-            # Unset all floatingIPs
-            # NOTE: FloatingIPs are not cleaned when filters are set
-            for obj in self.floating_ips():
-                self._service_cleanup_del_res(
-                    self.unset_floating_ip,
-                    obj,
-                    dry_run=dry_run,
-                    client_status_queue=client_status_queue,
-                    identified_resources=identified_resources,
-                    filters=filters,
-                    resource_evaluation_fn=resource_evaluation_fn,
-                )
+        :returns: :class:`~openstack.dns.v2.tld.TLD`
+        """
+        return self._find(_tld.TLD, name_or_id, ignore_missing=ignore_missing)
 
     # ====== TSIG keys ======
     def tsigkeys(self, **query):
@@ -891,3 +928,164 @@ class Proxy(proxy.Proxy):
         return self._find(
             _tsigkey.TSIGKey, name_or_id, ignore_missing=ignore_missing
         )
+
+    # ======== Blacklists ========
+    def blacklists(self, **query):
+        """Retrieve a generator of blacklists
+
+        :returns: A generator of blacklist
+            (:class:`~openstack.dns.v2.blacklist.Blacklist`) instances
+        """
+        return self._list(_blacklist.Blacklist, **query)
+
+    def get_blacklist(self, blacklist):
+        """Get a blacklist
+
+        :param blacklist: The value can be the ID of a blacklist
+            or a :class:`~openstack.dns.v2.blacklist.Blacklist` instance.
+
+        :returns: Blacklist instance.
+        :rtype: :class:`~openstack.dns.v2.blacklist.Blacklist`
+        """
+        return self._get(_blacklist.Blacklist, blacklist)
+
+    def create_blacklist(self, **attrs):
+        """Create a new blacklist
+
+        :param attrs: Keyword arguments which will be used to create
+            a :class:`~openstack.dns.v2.blacklist.Blacklist`,
+            comprised of the properties on the Blacklist class.
+
+        :returns: The results of blacklist creation.
+        :rtype: :class:`~openstack.dns.v2.blacklist.Blacklist`
+        """
+        return self._create(_blacklist.Blacklist, prepend_key=False, **attrs)
+
+    def update_blacklist(self, blacklist, **attrs):
+        """Update blacklist attributes
+
+        :param blacklist: The id or an instance of
+            :class: `~openstack.dns.v2.blacklist.Blacklist`.
+        :param attrs: attributes for update on
+            :class: `~openstack.dns.v2.blacklist.Blacklist`.
+
+        :rtype: :class: `~openstack.dns.v2.blacklist.Blacklist`.
+        """
+        return self._update(_blacklist.Blacklist, blacklist, **attrs)
+
+    def delete_blacklist(self, blacklist, ignore_missing=True):
+        """Delete a blacklist
+
+        :param blacklist: The id or an instance of
+            :class: `~openstack.dns.v2.blacklist.Blacklist`.
+
+        :returns: Blacklist been deleted
+        :rtype: :class:`~openstack.dns.v2.blacklist.Blacklist`
+        """
+        return self._delete(
+            _blacklist.Blacklist, blacklist, ignore_missing=ignore_missing
+        )
+
+    # ========== Utilities ==========
+    def wait_for_status(
+        self,
+        res: resource.ResourceT,
+        status: str,
+        failures: list[str] | None = None,
+        interval: int | float | None = 2,
+        wait: int | None = None,
+        attribute: str = 'status',
+        callback: ty.Callable[[int], None] | None = None,
+    ) -> resource.ResourceT:
+        """Wait for the resource to be in a particular status.
+
+        :param session: The session to use for making this request.
+        :param resource: The resource to wait on to reach the status. The
+            resource must have a status attribute specified via ``attribute``.
+        :param status: Desired status of the resource.
+        :param failures: Statuses that would indicate the transition
+            failed such as 'ERROR'. Defaults to ['ERROR'].
+        :param interval: Number of seconds to wait between checks.
+        :param wait: Maximum number of seconds to wait for transition.
+            Set to ``None`` to wait forever.
+        :param attribute: Name of the resource attribute that contains the
+            status.
+        :param callback: A callback function. This will be called with a single
+            value, progress. This is API specific but is generally a percentage
+            value from 0-100.
+
+        :return: The updated resource.
+        :raises: :class:`~openstack.exceptions.ResourceTimeout` if the
+            transition to status failed to occur in ``wait`` seconds.
+        :raises: :class:`~openstack.exceptions.ResourceFailure` if the resource
+            transitioned to one of the states in ``failures``.
+        :raises: :class:`~AttributeError` if the resource does not have a
+            ``status`` attribute
+        """
+        return resource.wait_for_status(
+            self, res, status, failures, interval, wait, attribute, callback
+        )
+
+    def wait_for_delete(
+        self,
+        res: resource.ResourceT,
+        interval: int = 2,
+        wait: int = 120,
+        callback: ty.Callable[[int], None] | None = None,
+    ) -> resource.ResourceT:
+        """Wait for a resource to be deleted.
+
+        :param res: The resource to wait on to be deleted.
+        :param interval: Number of seconds to wait before to consecutive
+            checks.
+        :param wait: Maximum number of seconds to wait before the change.
+        :param callback: A callback function. This will be called with a single
+            value, progress, which is a percentage value from 0-100.
+
+        :returns: The resource is returned on success.
+        :raises: :class:`~openstack.exceptions.ResourceTimeout` if transition
+            to delete failed to occur in the specified seconds.
+        """
+        return resource.wait_for_delete(self, res, interval, wait, callback)
+
+    def _get_cleanup_dependencies(self):
+        # DNS may depend on floating ip
+        return {'dns': {'before': ['network']}}
+
+    def _service_cleanup(
+        self,
+        dry_run=True,
+        client_status_queue=False,
+        identified_resources=None,
+        filters=None,
+        resource_evaluation_fn=None,
+        skip_resources=None,
+    ):
+        if not self.should_skip_resource_cleanup("zone", skip_resources):
+            # Delete all zones
+            for obj in self.zones():
+                self._service_cleanup_del_res(
+                    self.delete_zone,
+                    obj,
+                    dry_run=dry_run,
+                    client_status_queue=client_status_queue,
+                    identified_resources=identified_resources,
+                    filters=filters,
+                    resource_evaluation_fn=resource_evaluation_fn,
+                )
+
+        if not self.should_skip_resource_cleanup(
+            "floating_ip", skip_resources
+        ):
+            # Unset all floatingIPs
+            # NOTE: FloatingIPs are not cleaned when filters are set
+            for obj in self.floating_ips():
+                self._service_cleanup_del_res(
+                    self.unset_floating_ip,
+                    obj,
+                    dry_run=dry_run,
+                    client_status_queue=client_status_queue,
+                    identified_resources=identified_resources,
+                    filters=filters,
+                    resource_evaluation_fn=resource_evaluation_fn,
+                )
