@@ -10,7 +10,11 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-from openstack.identity import identity_service
+import typing as ty
+
+from keystoneauth1 import adapter
+import typing_extensions as ty_ext
+
 from openstack import resource
 
 
@@ -18,9 +22,6 @@ class Version(resource.Resource):
     resource_key = 'version'
     resources_key = 'versions'
     base_path = '/'
-    service = identity_service.IdentityService(
-        version=identity_service.IdentityService.UNVERSIONED
-    )
 
     # capabilities
     allow_list = True
@@ -31,9 +32,21 @@ class Version(resource.Resource):
     updated = resource.Body('updated')
 
     @classmethod
-    def list(cls, session, paginated=False, **params):
-        resp = session.get(cls.base_path,
-                           params=params)
+    def list(
+        cls,
+        session: adapter.Adapter,
+        paginated: bool = True,
+        base_path: str | None = None,
+        allow_unknown_params: bool = False,
+        *,
+        microversion: str | None = None,
+        headers: dict[str, str] | None = None,
+        **params: ty.Any,
+    ) -> ty.Generator[ty_ext.Self, None, None]:
+        if base_path is None:
+            base_path = cls.base_path
+
+        resp = session.get(base_path, params=params)
         resp = resp.json()
         for data in resp[cls.resources_key]['values']:
             yield cls.existing(**data)
