@@ -36,7 +36,8 @@ from openstack.identity.v3 import user
 from openstack.tests.unit import test_proxy_base
 
 USER_ID = 'user-id-' + uuid.uuid4().hex
-ENDPOINT_ID = 'user-id-' + uuid.uuid4().hex
+ENDPOINT_ID = 'endpoint-id-' + uuid.uuid4().hex
+PROJECT_ID = 'project-id-' + uuid.uuid4().hex
 
 
 class TestIdentityProxyBase(test_proxy_base.TestProxyBase):
@@ -115,7 +116,7 @@ class TestIdentityProxyDomainConfig(TestIdentityProxyBase):
             ignore_missing=False,
             method_args=['domain_id'],
             method_kwargs={},
-            expected_args=[],
+            expected_args=[None],
             expected_kwargs={
                 'domain_id': 'domain_id',
             },
@@ -128,7 +129,7 @@ class TestIdentityProxyDomainConfig(TestIdentityProxyBase):
             ignore_missing=True,
             method_args=['domain_id'],
             method_kwargs={},
-            expected_args=[],
+            expected_args=[None],
             expected_kwargs={
                 'domain_id': 'domain_id',
             },
@@ -157,7 +158,7 @@ class TestIdentityProxyDomainConfig(TestIdentityProxyBase):
             domain_config.DomainConfig,
             method_args=['domain_id'],
             method_kwargs={},
-            expected_args=[],
+            expected_args=[None],
             expected_kwargs={
                 'domain_id': 'domain_id',
             },
@@ -185,6 +186,14 @@ class TestIdentityProxyEndpoint(TestIdentityProxyBase):
     def test_endpoints(self):
         self.verify_list(self.proxy.endpoints, endpoint.Endpoint)
 
+    def test_project_endpoints(self):
+        self.verify_list(
+            self.proxy.project_endpoints,
+            endpoint.ProjectEndpoint,
+            method_kwargs={'project': PROJECT_ID},
+            expected_kwargs={'project_id': PROJECT_ID},
+        )
+
     def test_endpoint_update(self):
         self.verify_update(self.proxy.update_endpoint, endpoint.Endpoint)
 
@@ -314,6 +323,22 @@ class TestIdentityProxyProject(TestIdentityProxyBase):
     def test_project_update(self):
         self.verify_update(self.proxy.update_project, project.Project)
 
+    def test_project_associate_endpoint(self):
+        self._verify(
+            'openstack.identity.v3.project.Project.associate_endpoint',
+            self.proxy.associate_endpoint_with_project,
+            method_args=['project_id', 'endpoint_id'],
+            expected_args=[self.proxy, 'endpoint_id'],
+        )
+
+    def test_project_disassociate_endpoint(self):
+        self._verify(
+            'openstack.identity.v3.project.Project.disassociate_endpoint',
+            self.proxy.disassociate_endpoint_from_project,
+            method_args=['project_id', 'endpoint_id'],
+            expected_args=[self.proxy, 'endpoint_id'],
+        )
+
 
 class TestIdentityProxyService(TestIdentityProxyBase):
     def test_service_create_attrs(self):
@@ -360,6 +385,46 @@ class TestIdentityProxyUser(TestIdentityProxyBase):
     def test_user_update(self):
         self.verify_update(self.proxy.update_user, user.User)
 
+    def test_user_groups(self):
+        self.verify_list(
+            self.proxy.user_groups,
+            group.UserGroup,
+            method_kwargs={"user": 'user'},
+            expected_kwargs={"user_id": "user"},
+        )
+
+
+class TestIdentityProxyToken(TestIdentityProxyBase):
+    def test_token_validate(self):
+        self._verify(
+            "openstack.identity.v3.token.Token.validate",
+            self.proxy.validate_token,
+            method_args=['token'],
+            method_kwargs={'nocatalog': False, 'allow_expired': False},
+            expected_args=[self.proxy, 'token'],
+            expected_kwargs={'nocatalog': False, 'allow_expired': False},
+        )
+
+    def test_token_check(self):
+        self._verify(
+            "openstack.identity.v3.token.Token.check",
+            self.proxy.check_token,
+            method_args=['token'],
+            method_kwargs={'allow_expired': False},
+            expected_args=[self.proxy, 'token'],
+            expected_kwargs={'allow_expired': False},
+        )
+
+    def test_token_revoke(self):
+        self._verify(
+            "openstack.identity.v3.token.Token.revoke",
+            self.proxy.revoke_token,
+            method_args=['token'],
+            method_kwargs={},
+            expected_args=[self.proxy, 'token'],
+            expected_kwargs={},
+        )
+
 
 class TestIdentityProxyTrust(TestIdentityProxyBase):
     def test_trust_create_attrs(self):
